// This example creates a MIDI-file based on an algorithm.

import java.io.*;
import java.util.*;
import MIDICompose.*;
import MIDIComposeUtil.*;

public class AlgoMIDI implements MIDIVariables, GMVariables, ScaleVariables {
	public static void main(String args[])
	{
		String fileName = new String("AlgoMIDI.mid");					// type in the filename here
		FileOutputStream fpp = null;
		try { fpp = new FileOutputStream(new File(fileName)); }
		catch(IOException e) { System.out.println("Could not create file."); System.exit(0); }

		MIDIFile midi = new MIDIFile();												// creates a new MIDIFile
		//
		// start

		// ParameterSection START -----------------------------------------------------------

		double tempo = MODERATELY; 														// FAST, MODERATELY, SLOWLY

		int lead_octave = 5;
		int lead_inst = HARMONICA;														// HARMONICA, SAX, FLUTE, ...
		int carpet_inst = ORGAN;															// ORGAN, PIANO, STRING, ...
		int bass_inst = CONTRABASS;														// CONTRABASS, EBASS, ...

		int key = E;																					// C, SHARP+D, G+FLAT...
		int type = MINOR;																			// MINOR, MAJOR (or Scale.xxx)

		int bars = 12;																				// Song will be ... bars long

		// ParameterSection FIN -------------------------------------------------------------

		midi.setTempo(tempo);
		midi.append(new GMSystem(ENABLE));

		MIDIContainer lead = new MIDIContainer(CHAN_1);				// Container for Lead-Instrument
		MIDIContainer carpet = new MIDIContainer(CHAN_2);			// carpet(chord)-Instrument
		MIDIContainer bass = new MIDIContainer(CHAN_3);				// and BassoContinuum

		lead.append(new ProgramChange(lead_inst));						// sets instruments
		lead.append(new Rest(QUARTER));												// and a rest for progChange
		carpet.append(new ProgramChange(carpet_inst));
		carpet.append(new Rest(QUARTER));
		bass.append(new ProgramChange(bass_inst));
		bass.append(new Rest(QUARTER));

		Scale scale = new Scale(key,type);										// sets the Scale
		Chord acc = new Chord(scale);													// and the Chords for Scale
		Chord acc7 = new Chord(new Scale(key,MAJOR));					// V7 needs a MAJOR scale

		MIDIContainer mod = new MIDIContainer(CHAN_1);				// mod: a Modulation curve
		Ramp ramp = new Ramp(3*SIXTEENTH, new ControlChange(MODULATION,MIN), MAX/2);
		mod.append(ramp); ramp.turnRound(); ramp.setLength(SIXTEENTH); mod.append(ramp);

		Random rand = new Random();														// random-generator
		Chord accord;																					// the current Chord
		int degree; int mark;
		int keyNote = lead_octave*scale.octave();							// rootkey
		int oldNote = keyNote;
		int note = keyNote;																		// current Lead-note

		for(int i = 0; i < (bars-1); i++) {										// i counts bars
			degree = rand.nextInt() % (scale.octave() - 1);			// random degree
			if(degree < I) degree = -degree;
			if(degree == V) accord = acc7; else accord = acc;		// if V the V7 will be used

			bass.append(new Note(HALF, scale.pitch(degree,3)));	// bassoContinuum
			bass.append(new Note(HALF, scale.pitch(degree,3)));
			carpet.append(accord.notes(WHOLE, degree));					// chord for degree

			for(int s = 0; s < 4; s++) {												// the 4 beats for lead
				mark = lead.position();														// current pos is marked
				switch(rand.nextInt() % 6) {											// random step selected
					case 5: note += VI; break;
					case 4: note += V; break;
					case 3: case 2: case 1: note += II; break;
					case -3: case -2: case -1: note -= II; break;
					case -4: note -= IV; break;
					case -5: note -= V; break; }

				if((keyNote - note) > scale.octave()) 						// pitch correction for lead
					note += scale.octave();
				else if((note - keyNote) > scale.octave())
					note -= scale.octave();

				if(note==oldNote) lead.append(new Rest(QUARTER));	// same notes are rested
				else lead.append(new Note(QUARTER, scale.pitch(note)));
				if((rand.nextInt() % 5) == 4)
					lead.appendAt(mod, mark);												// random MOD-effects
				oldNote = note; }
		}

		bass.append(new Note(WHOLE, scale.pitch(I,3)));				// song ends with tonica
		carpet.append(acc.notes(WHOLE, I));
		lead.append(new Note(WHOLE, scale.pitch(I,5)));

		midi.appendAt(bass, START);
		midi.appendAt(carpet, START);
		midi.appendAt(lead, START);

		// end
		//
		try { midi.print(fpp); fpp.close(); }
		catch(IOException exception) {
			System.out.println("Could not write or close file."); System.exit(0); }
	}
}
