// This example shows you, how to handle the package MIDIComposeUtil

import java.io.*;
import MIDICompose.*;
import MIDIComposeUtil.*;

public class Compose implements MIDIVariables, GMVariables, ScaleVariables {
	public static void main(String args[])
	{
		String fileName = new String("Compose.mid");					// type in the filename here
		FileOutputStream fpp = null;
		try { fpp = new FileOutputStream(new File(fileName)); }
		catch(IOException e) { System.out.println("Could not create file."); System.exit(0); }

		MIDIFile midi = new MIDIFile();												// creates a new MIDIFile
		//
		// start

		midi.setTempo(FAST);																	// sets a fast tempo

		midi.append(new GMSystem(ENABLE));										// GeneralMIDI is enabled
		midi.append(new ProgramChange(0, CHAN_1, SAX));				// CHAN_1 is a Saxophone
		midi.append(new ProgramChange(0, CHAN_2, PIANO));			// CHAN_2 is a Piano

		MIDIContainer melodie = new MIDIContainer();					// Container for the notes

		Scale c = new Scale(C,MAJOR);													// song is in C-MAJOR
		Note note = new Note(QUARTER, c.pitch(0,5));					// c3 with a quarter of length
		Rest rest = new Rest(QUARTER);												// quarter-rest

		melodie.append(note);																	// 1:1		c3
		note.setPitch(c.pitch(2,5));													//        3rd note at 5th octave
		melodie.append(note);																	// 1:2		e3
		melodie.append(rest);																	// 1:3
		note.setPitch(c.pitch(4,5));
		melodie.append(note);																	// 1:4		g3

		note.setPitch(c.pitch(3,5));
		melodie.append(note);																	// 2:1		f3
		note.setPitch(c.pitch(1,5));
		melodie.append(note);																	// 2:2		d3
		melodie.append(rest);																	// 2:3
		note.setPitch(c.pitch(6,4));
		melodie.append(note);																	// 2:4		b2

		int mark = melodie.position();												// marks beginning of 3:1

		note.setPitch(c.pitch(0,5));
		note.setLength(WHOLE);																// length is now a whole
		melodie.append(note);																	// 3			c3

		ControlChange mod;
		mod = new ControlChange(MODULATION,MIN);							// Modulation-controller
		Ramp ramp = new Ramp(HALF + QUARTER, mod, MAX);				// from 0-127 in 3 beats

		melodie.appendAt(ramp, mark);													//   3:1 2 3		mod to 127
		ramp.turnRound();																			// ramp is turned round
		ramp.setLength(QUARTER);															// and shortened to 1 beat
		melodie.append(ramp);																	//   3:4				back to 0

		Chord accord = new Chord(c);													// Chord for Scale c
		accord.setChannel(CHAN_2);														// Chord set to Channel 2
		melodie.appendAt(accord.notes(HALF,I), START);				// tonica
		melodie.append(accord.notes(HALF,IV));								// subdominante
		melodie.append(accord.notes(HALF,V));									// dominante
		melodie.append(accord.notes(HALF,V, 4, 4));						// added (with 7) to melodie
		melodie.append(accord.notes(WHOLE,I));								// tonica

		midi.appendAt(melodie, QUARTER);											// melodie is added to MIDIfile

		midi.append(new Rest(HALF));													// just for demonstration
		//melodie.setType(melodie.LINKED);
		//melodie.setType(melodie.ABSOLUTE);
		//melodie.setChannelable(CHAN_2);
		melodie.turnRound();
		melodie.setLength(melodie.length()-WHOLE);
		midi.append(melodie);

		// end
		//
		try { midi.print(fpp); fpp.close(); }
		catch(IOException e) {
			System.out.println("Could not write or close file."); System.exit(0); }
	}
}
