// This file was created in lesson 'docs/MIDICompose/MyCompose01.html'

import java.io.*;
import MIDICompose.*;

class Envelope extends MIDIContainer
{
  Envelope(int length) { this(length/8,length-length/4,length/8); }
  Envelope(int attackLength, int sustainLength, int releaseLength) {
    super();
    MasterVolume vol = new MasterVolume(0);
    Ramp ramp = new Ramp(attackLength, vol, vol.maxAmount());
    append(ramp);
    append(new Rest(sustainLength));
    ramp.turnRound();
    ramp.setLength(releaseLength);
    append(ramp);
    lock(); }

  protected Envelope(Envelope object) { super(object); }
  public MIDIObject copy() { return new Envelope(this); }
}

class TremoloNote extends MIDIContainer implements MIDIVariables
{
  TremoloNote(int length, int pitch) { this(length, pitch, SIXTEENTH); }
  TremoloNote(int length, int pitch, int tremoloLength) {
    super(0, LINKED, CHAN_1);
    int number = length / tremoloLength;
    for(int i = 0; i < number; i++)
      append(new Note(tremoloLength, pitch + (i%2) * OCTAVE));
    lock(); }

  protected TremoloNote(TremoloNote object) { super(object); }
  public MIDIObject copy() { return new TremoloNote(this); }
}

public class MyCompose implements MIDIVariables, GMVariables
{
  public static void main(String args[])
  {
    String fileName = new String("MyCompose.mid");

    FileOutputStream fpp = null;
    try { fpp = new FileOutputStream(new File(fileName)); }
    catch(IOException e) {
      System.out.println("Could not create file."); System.exit(0); }

    MIDIFile midi = new MIDIFile();
    midi.setTempo(FAST);

    midi.append(new GMSystem(ENABLE));
    midi.append(new ProgramChange(0, CHAN_1, OBOE));
    midi.append(new ProgramChange(0, CHAN_2, CLARINET));

    CombinedControlChange amountObject = new CombinedControlChange(MODULATION, 0);
    amountObject.setAmount(amountObject.minAmount());

    Ramp ramp = new Ramp(8*WHOLE, amountObject, amountObject.maxAmount());

    int pos = midi.position();
    midi.append(ramp);
    midi.setPosition(pos);

    midi.append(new Rest(SIXTEENTH));

    MIDIContainer melody = new MIDIContainer();

    melody.append(new Note(QUARTER, C + OCTAVE * 5));
    melody.append(new Note(QUARTER, D + OCTAVE * 5));
    melody.append(new Note(QUARTER, E + OCTAVE * 5));
    melody.append(new Note(QUARTER, F + OCTAVE * 5));
    melody.append(new Note(HALF,    G + OCTAVE * 5));
    melody.append(new TremoloNote(HALF, G + OCTAVE * 5));

    melody.append(new Note(QUARTER, A + OCTAVE * 5));
    melody.append(new Note(QUARTER, A + OCTAVE * 5));
    melody.append(new Note(QUARTER, A + OCTAVE * 5));
    melody.append(new Note(QUARTER, A + OCTAVE * 5));
    melody.append(new Note(WHOLE,   G + OCTAVE * 5));

    melody.append(new Note(QUARTER, A + OCTAVE * 5));
    melody.append(new Note(QUARTER, A + OCTAVE * 5));
    melody.append(new Note(QUARTER, A + OCTAVE * 5));
    melody.append(new Note(QUARTER, A + OCTAVE * 5));
    melody.append(new TremoloNote(WHOLE, G + OCTAVE * 5));

    melody.append(new Note(QUARTER, F + OCTAVE * 5));
    melody.append(new Note(QUARTER, F + OCTAVE * 5));
    melody.append(new Note(QUARTER, F + OCTAVE * 5));
    melody.append(new Note(QUARTER, F + OCTAVE * 5));
    melody.append(new Note(HALF,    E + OCTAVE * 5));
    melody.append(new TremoloNote(HALF, E + OCTAVE * 5));

    melody.append(new Note(QUARTER, D + OCTAVE * 5));
    melody.append(new Note(QUARTER, D + OCTAVE * 5));
    melody.append(new Note(QUARTER, D + OCTAVE * 5));
    melody.append(new Note(QUARTER, D + OCTAVE * 5));
    melody.append(new Note(HALF,    C + OCTAVE * 5));

    MIDIContainer melodyCopy = (MIDIContainer)melody.copy();
    melodyCopy.setLength(melody.length() / 2);
    midi.append(melodyCopy);
    midi.append(new Rest(QUARTER));
    int mark = midi.position();
    midi.append(melody);
    melody.turnRound();
    melody.setChannelable(CHAN_2);
    midi.appendAt(melody, mark);

		midi.appendAt(new Envelope(midi.length()), START);

    try { midi.print(fpp); fpp.close(); }
    catch(IOException e) {
      System.out.println("Could not write or close file."); System.exit(0); }                                  // program stops
  }
}
