package MIDICompose;

public class ControlChange extends VoiceEvent implements Channelable, Amountable
{
	public static final int MIN = 0;
	public static final int MAX = 127;
	public int ZERO = MIN;																		// depending on controller
	public static final int CENTER = 64;											// for PAN and BALANCE

	public static final int BANK_SELECT = 0;									// CombineControlChange (32)
	public static final int MODULATION_WHEEL = 1;							// CombineControlChange (33)
	public static final int MODULATION = 1;
	public static final int BREATH_CONTROLLER = 2;						// CombineControlChange (34)
	public static final int BREATH = 2;
	public static final int FOOT_PEDAL = 4;										// CombineControlChange (36)
	public static final int PORTAMENTO_TIME = 5;							// CombineControlChange (37)
	public static final int DATA_ENTRY = 6;										// CombineControlChange (38)
	public static final int VOLUME = 7;												// CombineControlChange (39)
	public static final int BALANCE = 8;											// CombineControlChange (40)
	public static final int PAN_POSITION = 10;								// CombineControlChange (42)
	public static final int PAN = 10;
	public static final int EXPRESSION = 11;									// CombineControlChange (43)
	public static final int EFFECT_CONTROL_1 = 12;						// CombineControlChange (44)
	public static final int EFFECT1 = 12;
	public static final int EFFECT_CONTROL_2 = 13;						// CombineControlChange (45)
	public static final int EFFECT2 = 13;
	public static final int GENERAL_PURPOSE_SLIDER_1 = 16;
	public static final int GENERAL_PURPOSE_SLIDER_2 = 17;
	public static final int GENERAL_PURPOSE_SLIDER_3 = 18;
	public static final int GENERAL_PURPOSE_SLIDER_4 = 19;
	public static final int HOLD_PEDAL = 64;									// (ON/OFF)
	public static final int PORTAMENTO = 65;									// (ON/OFF)
	public static final int SUSTENUTO_PEDAL = 66;							// (ON/OFF)
	public static final int SOFT_PEDAL = 67;									// (ON/OFF)
	public static final int LEGATO_PEDAL = 68;								// (ON/OFF)
	public static final int HOLD_2_PEDAL = 69;								// (ON/OFF)
	public static final int SOUND_VARIATION = 70;
	public static final int SOUND_TIMBRE = 71;
	public static final int SOUND_RELEASE_TIME = 72;
	public static final int SOUND_ATTACK_TIME = 73;
	public static final int SOUND_BRIGHTNESS = 74;
	public static final int SOUND_CONTROL_6 = 75;
	public static final int SOUND_CONTROL_7 = 76;
	public static final int SOUND_CONTROL_8 = 77;
	public static final int SOUND_CONTROL_9 = 78;
	public static final int SOUND_CONTROL_10 = 79;
	public static final int GENERAL_PURPOSE_BUTTON_1 = 80;		// (ON/OFF)
	public static final int GENERAL_PURPOSE_BUTTON_2 = 81;		// (ON/OFF)
	public static final int GENERAL_PURPOSE_BUTTON_3 = 82;		// (ON/OFF)
	public static final int GENERAL_PURPOSE_BUTTON_4 = 83;		// (ON/OFF)
	public static final int EFFECTS_LEVEL = 91;
	public static final int TREMULO_LEVEL = 92;
	public static final int CHORUS_LEVEL = 93;
	public static final int CELESTE_LEVEL = 94;
	public static final int PHASER_LEVEL = 95;
	public static final int DATA_BUTTON_INCREMENT = 96;				// no amount
	public static final int DATA_BUTTON_DECREMENT = 97;				// no amount
	public static final int NON_REGISTERED_PARAMETER = 99; 		// CombineControlChange (98)
	public static final int REGISTERED_PARAMETER = 101;				// CombineControlChange (100)
	public static final int ALL_SOUND_OFF = 120;							// no amount
	public static final int ALL_CONTROLLERS_OFF = 121;				// no amount
	public static final int LOCAL_KEYBOARD = 122; 						// (ON/OFF)
	public static final int ALL_NOTES_OFF = 123;							// no amount
	public static final int OMNI_MODE_OFF = 124;							// no amount
	public static final int OMNI_MODE_ON = 125;								// no amount
	public static final int MONO_OPERATION = 126;							// complex amount
	public static final int POLY_OPERATION = 127;							// no amount

	public ControlChange(int controller, int amount) { this(0, 0, controller, amount); }
	public ControlChange(int deltaTime, int channel, int controller, int amount) {
		super(deltaTime, 0xb0, channel);
		setController(controller);
		setZero(controller);
		setAmount(amount);
		lock(); }

	public int controller() { return dataValueAt(0); }
	public void setController(int controller) { setDataValueAt(controller,0); }

	private void setZero(int controller) {
		ZERO = MIN;
		switch(controller) {
			case VOLUME: ZERO = MAX; break;
			case PAN:
			case BALANCE:
			case PORTAMENTO_TIME: ZERO = CENTER;
		}}

	public int amount() { return dataValueAt(1); }
	public void setAmount(int amount) { setDataValueAt(amount,1); }

	public int minAmount() { return MIN; }
	public int maxAmount() { return MAX; }
	public int zeroAmount() { return ZERO; }

	public MIDIObject copy() {
		return new ControlChange(deltaTime(), channel(), controller(), amount()); }
}

