package MIDICompose;

public class Ramp extends MIDIContainer implements Channelable, Lengthable
{
	public static final int LINEAR = 1;		// 0  1  2  3  4  5  6  7  8  9
	public static final int SOFT = 2;			// 0   1   2  3 45 6  7   8   9
	public static final int EXP = 3;			// 0       1     2   3  4 56789
	public static final int LOG = 4;			// 01234 5  6   7     8       9

	private int curve;													// stores ramp-type

	public Ramp(int length, Amountable object, int end) {
		this(length, object, end, SOFT, length / MIDIVariables.SIXTEENTH); }
	public Ramp(int length, Amountable object, int end, int curve, int steps) {
		this(0, -1, length, object, end, curve, steps); }
	public Ramp(int deltaTime, int channel, int length, Amountable object, int end, int curve, int steps) {
		super(deltaTime, FREE);
		Amountable o = (Amountable)object.copy();	// copy of object is used to create the ramp
		if(o instanceof Channelable) setChannelable(((Channelable)o).channel());
		if(channel != -1) setChannel(channel);		// if channel is set in constructor
		int start = o.amount();
		for(int i = 0; i <= steps; i++) {
			o.setAmount(start + (int)Math.round(i*((end - start)/(double)steps)));
			o.setDeltaTime(length);
			add(o.copy()); }												// adds a new object with the next value
		setCurve(curve);													// sets the deltaTimes concerning to curve
		lock(); }
	protected Ramp(Ramp object) { super(object); setCurve(object.curve()); } // for copy()

	public int curve() { return curve; }
	public void setCurve(int curve) {					// sets the curve
		this.curve = curve;
		for(int i = 0; i < size(); i++) setDeltaTimeAt(deltaTimeFor(i), i); }

	private int deltaTimeFor(int current) {		// returns the deltaTime for each element
		double factor;
		double steps = (double)size() - 1;
		int length = length();
		switch(curve) {
			case LINEAR:
					return (int)Math.round(current * (length / steps));
			case SOFT:
					factor = Math.acos((2 / steps) * current - 1) / Math.PI;
					return (int)Math.round(length - (factor * length));
			case EXP:
					factor = Math.log((current / steps) * (Math.exp(5) - 1) + 1) / 5;
					return (int)Math.round(factor * length);
			case LOG:
					factor = (Math.exp(5 * (current / steps)) - 1) / (Math.exp(5) - 1);
					return (int)Math.round(factor * length);
			default:
					System.out.println("ERROR: unknown Ramp-type."); System.exit(-1); return -1;
	}}

	public MIDIObject copy() { return new Ramp(this); }
}
