package MIDICompose;

import java.io.*;

public class SystemExclusive extends MIDIEvent
{
	public static final int SYSEX = 0xf0;
	public static final int ESCAPED = 0xf7;

	private int type;

	public SystemExclusive() { this(0, SYSEX); }
	public SystemExclusive(int deltaTime, int type) {
		super(deltaTime, type); setType(type); }

	private void setType(int type) {
		this.type = type;
		if((type != SYSEX) && (type != ESCAPED)) {
			System.out.println("ERROR: wrong SystemExclusive-type."); System.exit(-1);
		}}

	public int dataValueAt(int index) { return super.dataValueAt(index); }
	public void setDataValueAt(int value, int index) {		// overwritten for ESCAPED
		if(((value > 0x7f) && ((type != ESCAPED) || (index != 0))) || (value < 0)) {
			System.out.println("ERROR: data-value out of range."); System.exit(-1); }
		if(index >= 0) index++;
		if(size() == index) add(new MIDIByte(value));
		else if(size() > index) ((MIDIByte)elementAt(index)).setValue(value);
		else { System.out.println("ERROR: data-index out of range."); System.exit(-1); }}

	int printSize() {
		int bytes = 1;
		if(size() < 0x80) bytes++;
		else if(size() < 0x4000) bytes += 2;
		else if(size() < 0x200000) bytes += 3;
		else bytes += 4;
		return bytes + super.printSize(); }

	void print(FileOutputStream fpp) throws IOException {	// overwritten
		printVarLen(fpp, writeTime());
		fpp.write(type);
		printVarLen(fpp, size());
		for(int i = 1; i < size(); i++) fpp.write(((MIDIByte)elementAt(i)).value());
		fpp.write(0xf7); }

	public MIDIObject copy() {
		SystemExclusive object = new SystemExclusive(deltaTime(), type);
		for(int i = 0; i < (size() - 1); i++) object.setDataValueAt(dataValueAt(i),i);
		return object; }
}
